# A simple class to create and manage locally maintained repositories using
# tar file containing the RPMs.
#
# @summary Creates and hosts the specified RPM repositories.
#
# @param mirror_fqdn
#   Fully Qualified Domain Name which will be used as the apache vhost name on
#   this server. Note that this FQDN should resolve to this server.
#
# @param base_url_default
#   This specifies the URL from which the repo tar should be extracted or the
#   remote files should be replicated if a specific url is not provided in
#   repo_list. When used, the application assumes the source data is located
#   within the specified directory.
#
#   For example, given a repo named 'test-repo' at version '1.0.5.1':
#     * `http://bin.example.org/data` - expects http://bin.example.org/data/test-repo-1.0.5.1.tar to exist
#     * `puppet:///repo_files` - expects the folder structure 'puppet:///repo_files/test-repo/1.0.5.1' 
#
# @param repo_base_path
#   Absolute path to the parent directory of the yum repository
#       
# @param repo_base_cache_path
#   Absolute path to the parent directory of the yum repository cache
#   directory.
#
# @param repo_list
#   A hash where the keys are the names of the repos to be deployed. The
#   following options are supported for each repo:
#   * version        - version of the source repo file
#   * source_prefix  - prefix of the file, defaults to the repo name.
#
#   @note At least one repository must be specified
#
# @param suppress_cron_stdout
#   When enabled stdout generated by the createrepo command(s) will be directed
#   to `/dev/null`. This is very helpful if these messages would be emailed
#   unnecessarily in your environment.
#
# @example Definition of three repositories
#   class { 'local_rpm_repo':
#     base_url_default => 'http://bin.example.com/data/',
#     mirror_fqdn      => 'yum.example.com',
#     repo_list        => {
#       'repo1' => { version => '1.0.0.0' },
#       'repo2' => { version => '1.0.0.0' },
#       'repo3' => { version => '1.0.0.0' },
#     },
#   }
class local_rpm_repo (
  Stdlib::Fqdn $mirror_fqdn,
  Optional[Local_rpm_repo::SourceUrl] $base_url_default,
  Stdlib::AbsolutePath $repo_base_path              = '/var/yumrepos',
  Stdlib::AbsolutePath $repo_base_cache_path        = '/var/cache/yumrepos',
  Hash[String, Local_rpm_repo::RepoData] $repo_list = {},
  Boolean $suppress_cron_stdout                     = true,
) {
  # Ensure Apache is installed
  include apache

  # Fail if no repo is specified
  if empty($repo_list) {
    fail('At least one repo must be specified!')
  }

  # Ensure the parent directories exist
  # The default paths are described here
  # https://forge.puppet.com/palli/createrepo#usage
  file {
    default:
      ensure => 'directory',
      owner  => 'root',
      group  => 'root',
      mode   => '0755',
      ;
    $repo_base_path:
      ;
    $repo_base_cache_path:
      ;
  }

  # Configure a web-server for the repository
  apache::vhost { $mirror_fqdn:
    port    => '80',
    docroot => $repo_base_path,
    require => File[$repo_base_path],
  }

  # Iterate through the list of repos and create each one
  $repo_list.each |$repo_name, $attributes| {
    # Generate the repo structure
    createrepo { $repo_name:
      suppress_cron_stdout => $suppress_cron_stdout,
      require              => [
        File[$repo_base_path],
        File[$repo_base_cache_path],
      ]
    }

    # Determine the source type
    if 'base_url' in $attributes {
      $base_url = $attributes['base_url']
    } else {
      $base_url = $base_url_default
    }

    # Create the versions subdirectory if it's not there
    $versions_path = "${repo_base_path}/${repo_name}/versions"
    file { $versions_path:
      ensure  => 'directory',
      owner   => 'root',
      group   => 'root',
      mode    => '0755',
      require => Createrepo[$repo_name]
    }

    # Compose the parameters 
    $source_args = $attributes
      + { 'base_url' => $base_url }
      + { 'versions_path' => $versions_path }
    case $base_url {
      /^http/: {
        local_rpm_repo::tar_source { $repo_name:
          require => File[$versions_path],
          *       => $source_args,
        }
      }
      /^puppet/: {
        local_rpm_repo::puppet_source { $repo_name:
          require => File[$versions_path],
          *       => $source_args,
        }
      }
      default: {
        # Should be impossible to hit - here as a safety
        fail("Repo ${repo_name}: Invalid base_url ${base_url}!")
      }
    }
  }
}
